@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 #
-# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2013 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -33,7 +33,7 @@ relevant aspects of the application being represented.
 """
 __docformat__ = 'reStructuredText'
 
-__version__ = '2.0.5 version (SVN $Revision: 3506 $)'
+__version__ = '2.1 version (SVN $Revision: 3697 $)'
 
 
 import copy
@@ -50,7 +50,6 @@ import logging
 import logging.config
 log = logging.getLogger("gc3.gc3libs")
 
-import gc3libs.exceptions
 from gc3libs.quantity import Memory, kB, MB, GB, Duration, hours, minutes, seconds
 from gc3libs.compat._collections import OrderedDict
 
@@ -85,6 +84,7 @@ class Default(object):
     SHELLCMD_LRMS = 'shellcmd'
     SLURM_LRMS = 'slurm'
     SUBPROCESS_LRMS = 'subprocess'
+    EC2_LRMS = 'ec2'
 
     # Transport information
     SSH_PORT = 22
@@ -104,7 +104,7 @@ class Default(object):
 
 from gc3libs.exceptions import *
 from gc3libs.persistence import Persistable
-import gc3libs.url
+from gc3libs.url import UrlKeyDict, UrlValueDict
 from gc3libs.utils import defproperty, deploy_configuration_file, Enum, History, Struct, safe_repr
 
 
@@ -786,8 +786,8 @@ class Application(Task):
 
         self.arguments = [ str(x) for x in arguments ]
 
-        self.inputs = Application._io_spec_to_dict(gc3libs.url.UrlKeyDict, inputs, True)
-        self.outputs = Application._io_spec_to_dict(gc3libs.url.UrlValueDict, outputs, False)
+        self.inputs = Application._io_spec_to_dict(UrlKeyDict, inputs, True)
+        self.outputs = Application._io_spec_to_dict(UrlValueDict, outputs, False)
 
         # check that remote entries are all distinct
         # (can happen that two local paths are mapped to the same remote one)
@@ -1949,6 +1949,55 @@ class Run(Struct):
             # regular exit
             return (0, rc)
 
+# Factory method to create an Engine instance
+
+def create_engine(*conf_files, **extra_args):
+    """
+    Create and return a `gc3libs.core.Engine`:class: class.
+
+    It accepts an optional list of configuration filenames. If the
+    filenames contain a `~` or a variable name, it will be expanded
+    automatically.
+
+    Called without arguments, a configuration file will be searched in
+    ~/.gc3/gc3pie.conf (and used if found).
+
+    :param conf_files:    List of configuration files to read.
+    :param store:         See the like-named argument to the `Engine`:class: constructor.
+    :param can_submit:    See the like-named argument to the `Engine`:class: constructor.
+    :param can_retrieve:  See the like-named argument to the `Engine`:class: constructor.
+    :param max_in_flight: See the like-named argument to the `Engine`:class: constructor.
+    :param max_submitted: See the like-named argument to the `Engine`:class: constructor.
+
+    Any extra keyword argument is passed unchanged to the
+    `Configuration`:class: constructor.
+
+    """
+    store = extra_args.pop('store', None)
+    can_submit = extra_args.pop('can_submit', True)
+    can_retrieve = extra_args.pop('can_retrieve', True)
+    max_in_flight = extra_args.pop('max_in_flight', 0)
+    max_submitted = extra_args.pop('max_submitted', 0)
+
+    from gc3libs.config import Configuration
+    from gc3libs.core import Core, Engine
+    conf_files = [
+        os.path.expandvars(
+            os.path.expanduser(fname)) for fname in conf_files]
+    if not conf_files:
+        conf_files = Default.CONFIG_FILE_LOCATIONS[:]
+
+    if not 'auto_enable_auth' in extra_args:
+        extra_args['auto_enable_auth'] = True
+
+    cfg = Configuration(*conf_files, **extra_args)
+    core = Core(cfg)
+    engine = Engine(core, store=store,
+                    can_submit=can_submit, can_retrieve=can_retrieve,
+                    max_in_flight=max_in_flight, max_submitted=max_submitted)
+
+    return engine
+
 
 ## main: run tests
 
