@@ -20,7 +20,7 @@ Job control on SGE clusters (possibly connecting to the front-end via SSH).
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.0.5 version (SVN $Revision$)'
+__version__ = '2.1 version (SVN $Revision$)'
 
 
 import datetime
@@ -375,7 +375,8 @@ class LsfLrms(batch.BatchSystem):
             self, name,
             architecture, max_cores, max_cores_per_job,
             max_memory_per_core, max_walltime, auth,
-            frontend, transport, accounting_delay, **extra_args)
+            frontend, transport, accounting_delay=accounting_delay,
+            **extra_args)
 
         self.bsub = self._get_command_argv('bsub')
 
