@@ -19,7 +19,7 @@ Top-level interface to Grid functionality.
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.0.5 version (SVN $Revision: 3506 $)'
+__version__ = '2.1 version (SVN $Revision: 3697 $)'
 __date__ = '$Date$'
 
 
@@ -94,14 +94,13 @@ specified in the configuration file.
           - or it can be a string: only resources whose name matches
             (wildcards ``*`` and ``?`` are allowed) are retained.
         """
-        try:
-            self._lrms = dict( (name,lrms) for (name,lrms) in self._lrms.iteritems()
-                               if match(lrms) )
-        except:
-            # `match` is not callable, then assume it's a
-            # glob pattern and select resources whose name matches
-            self._lrms = dict( (name,lrms) for (name,lrms) in self._lrms.iteritems()
-                               if fnmatch(lrms.name, match) )
+        for lrms in self._lrms.itervalues():
+            try:
+                if not match(lrms):
+                    lrms.enabled = False
+            except:
+                if not fnmatch(lrms.name, match):
+                    lrms.enabled = False
         return len(self._lrms)
 
 
@@ -135,8 +134,14 @@ specified in the configuration file.
 
         auto_enable_auth = extra_args.get('auto_enable_auth', self.auto_enable_auth)
 
-        lrms =  self.get_backend(app.execution.resource_name)
-        lrms.free(app)
+        if hasattr(app.execution, 'resource_name'):
+            lrms =  self.get_backend(app.execution.resource_name)
+            lrms.free(app)
+        else:
+            gc3libs.log.debug(
+                "Core.__free_application(): Application `%s` does not have an "
+                "`execution.resource_name` attribute. Assuming it has been "
+                "aborted before submission." % str(app))
 
     def __free_task(self, task, **extra_args):
         """Implementation of `free` on generic `Task` objects."""
@@ -178,7 +183,8 @@ specified in the configuration file.
         elif job.state != Run.State.NEW:
             return
 
-        if len(self._lrms) == 0:
+        lrms_list = filter(lambda x: x.enabled, self._lrms.itervalues())
+        if len(lrms_list) == 0:
             raise gc3libs.exceptions.NoResources(
                 "Could not initialize any computational resource"
                 " - please check log and configuration file.")
@@ -186,7 +192,7 @@ specified in the configuration file.
         gc3libs.log.debug('Performing brokering ...')
         # decide which resource to use
         # (Resource)[] = (Scheduler).PerformBrokering((Resource)[],(Application))
-        _selected_lrms_list = app.compatible_resources(self._lrms.itervalues())
+        _selected_lrms_list = app.compatible_resources(lrms_list)
         gc3libs.log.debug('Application scheduler returned %d matching resources',
                            len(_selected_lrms_list))
         if 0 == len(_selected_lrms_list):
@@ -221,7 +227,7 @@ specified in the configuration file.
         if len(_selected_lrms_list) == 0:
             raise gc3libs.exceptions.LRMSSubmitError(
                 "No computational resources left after brokering."
-                " Aborting submission of job '%s'", app)
+                " Aborting submission of job '%s'" % app)
 
         exs = [ ]
         # Scheduler.do_brokering returns a sorted list of valid lrms
@@ -234,6 +240,11 @@ specified in the configuration file.
                         % (lrms.name,
                            time.ctime(job.timestamp[Run.State.NEW])))
                 lrms.submit_job(app)
+            except gc3libs.exceptions.LRMSSkipSubmissionToNextIteration, ex:
+                gc3libs.log.info(
+                    "Submission of job %s delayed" % app)
+                exs.append(ex)
+                break
             except Exception, ex:
                 gc3libs.log.info(
                     "Error in submitting job to resource '%s': %s: %s",
@@ -364,11 +375,15 @@ specified in the configuration file.
 
             # XXX: Re-enabled the catch-all clause otherwise the loop stops at the first erroneous iteration
             except Exception, ex:
-                gc3libs.log.warning("Ignored error in Core.update_job_state(): %s", str(ex))
-                # print again with traceback at a higher log level
-                gc3libs.log.debug("Ignored error in Core.update_job_state(): %s: %s",
-                                  ex.__class__.__name__, str(ex), exc_info=True)
-                continue
+                if 'GC3PIE_NO_CATCH_ERRORS' in os.environ:
+                    # propagate generic exceptions for debugging purposes
+                    raise
+                else:
+                    gc3libs.log.warning("Ignored error in Core.update_job_state(): %s", str(ex))
+                    # print again with traceback at a higher log level
+                    gc3libs.log.debug("Ignored error in Core.update_job_state(): %s: %s",
+                                      ex.__class__.__name__, str(ex), exc_info=True)
+                    continue
 
     def __update_task(self, tasks, **extra_args):
         """Implementation of `update_job_state` on generic `Task` objects."""
@@ -447,8 +462,10 @@ specified in the configuration file.
             # clear previous data staging errors
             if job.signal == Run.Signals.DataStagingFailure:
                 job.signal = 0
-        except gc3libs.exceptions.InvalidResourceName, irn:
-            gc3libs.log.warning("Failed while retrieving resource %s from core.Detailed Error message: %s" % (app.execution.resource_name, str(irn)))
+        except gc3libs.exceptions.InvalidResourceName, ex:
+            gc3libs.log.warning(
+                "Failed retrieving resource %s from core."
+                " Detailed Error message: %s" % (app.execution.resource_name, str(ex)))
             ex = app.fetch_output_error(ex)
             if isinstance(ex, Exception):
                 job.info = ("No output could be retrieved: %s" % str(ex))
@@ -456,7 +473,8 @@ specified in the configuration file.
             else:
                 return
         except gc3libs.exceptions.RecoverableDataStagingError, rex:
-            job.info = ("Temporary failure when retrieving results: %s. Ignoring error, try again." % str(rex))
+            job.info = ("Temporary failure when retrieving results: %s."
+                        " Ignoring error, try again." % str(rex))
             return
         except gc3libs.exceptions.UnrecoverableDataStagingError, ex:
             job.signal = Run.Signals.DataStagingFailure
@@ -464,14 +482,10 @@ specified in the configuration file.
             if isinstance(ex, Exception):
                 job.info = ("No output could be retrieved: %s" % str(ex))
                 raise ex
-            else:
-                return
         except Exception, ex:
             ex = app.fetch_output_error(ex)
             if isinstance(ex, Exception):
                 raise ex
-            else:
-                return
 
         # successfully downloaded results
         gc3libs.log.debug("Downloaded output of '%s' (which is in state %s)"
@@ -519,6 +533,11 @@ specified in the configuration file.
         try:
             lrms = self.get_backend(job.resource_name)
             lrms.cancel_job(app)
+        except AttributeError:
+            # A job in state NEW does not have a `resource_name`
+            # attribute.
+            if job.state != Run.State.NEW:
+                raise
         except gc3libs.exceptions.InvalidResourceName, irn:
             gc3libs.log.warning("Failed while retrieving resource %s from core.Detailed Error message: %s" % (app.execution.resource_name, str(irn)))
         gc3libs.log.debug("Setting job '%s' status to TERMINATED"
@@ -533,7 +552,7 @@ specified in the configuration file.
                              " since task is being killed: %s",
                              str(ex))
         job.signal = Run.Signals.Cancelled
-        job.history.append("Cancelled.")
+        job.history.append("Cancelled")
 
     def __kill_task(self, task, **extra_args):
         extra_args.setdefault('auto_enable_auth', self.auto_enable_auth)
@@ -603,6 +622,8 @@ specified in the configuration file.
         """
         for lrms in self._lrms.itervalues():
             try:
+                if not lrms.enabled:
+                    continue
                 auto_enable_auth = extra_args.get('auto_enable_auth', self.auto_enable_auth)
                 resource = lrms.get_resource_status()
                 resource.updated = True
@@ -886,11 +907,15 @@ class Engine(object):
                 self._terminated.append(task)
                 transitioned.append(index)
             except Exception, x:
-                gc3libs.log.error("Ignored error in killing task '%s': %s: %s",
-                                  task, x.__class__.__name__, str(x))
-                # print again with traceback info at a higher log level
-                gc3libs.log.debug("Ignored error in killing task '%s': %s: %s",
-                                  task, x.__class__.__name__, str(x), exc_info=True)
+                if 'GC3PIE_NO_CATCH_ERRORS' in os.environ:
+                    # propagate generic exceptions for debugging purposes
+                    raise
+                else:
+                    gc3libs.log.error("Ignored error in killing task '%s': %s: %s",
+                                      task, x.__class__.__name__, str(x))
+                    # print again with traceback info at a higher log level
+                    gc3libs.log.debug("Ignored error in killing task '%s': %s: %s",
+                                      task, x.__class__.__name__, str(x), exc_info=True)
         # remove tasks that transitioned to other states
         for index in reversed(transitioned):
             del self._to_kill[index]
@@ -950,14 +975,21 @@ class Engine(object):
                             currently_submitted += 1
                             currently_in_flight += 1
                     except Exception, x:
-                        gc3libs.log.error("Ignored error in submitting task '%s': %s: %s",
-                                          task, x.__class__.__name__, str(x))
-                        # print again with traceback at a higher log level
-                        gc3libs.log.debug("Ignored error in submitting task '%s': %s: %s",
-                                          task, x.__class__.__name__, str(x), exc_info=True)
-                        # record the fact in the task's history
-                        task.execution.history("Submission failed: %s: %s"
-                                               % (x.__class__.__name__, str(x)))
+                        if 'GC3PIE_NO_CATCH_ERRORS' in os.environ:
+                            # propagate generic exceptions for debugging purposes
+                            raise
+                        else:
+                            gc3libs.log.error(
+                                "Ignored error in submitting task '%s': %s: %s",
+                                task, x.__class__.__name__, str(x))
+                            # print again with traceback at a higher log level
+                            gc3libs.log.debug(
+                                "Ignored error in submitting task '%s': %s: %s",
+                                task, x.__class__.__name__, str(x), exc_info=True)
+                            # record the fact in the task's history
+                            task.execution.history(
+                                "Submission failed: %s: %s"
+                                % (x.__class__.__name__, str(x)))
                 index += 1
         # remove tasks that transitioned to SUBMITTED state
         for index in reversed(transitioned):
@@ -983,10 +1015,16 @@ class Engine(object):
                     task.execution.state = Run.State.TERMINATED
                     task.changed = True
                 except Exception, x:
-                    gc3libs.log.error("Ignored error in fetching output of task '%s': %s: %s",
-                                      task, x.__class__.__name__, str(x))
-                    gc3libs.log.debug("Ignored error in fetching output of task '%s': %s: %s",
-                                      task, x.__class__.__name__, str(x), exc_info=True)
+                    if 'GC3PIE_NO_CATCH_ERRORS' in os.environ:
+                        # propagate generic exceptions for debugging purposes
+                        raise
+                    else:
+                        gc3libs.log.error(
+                            "Ignored error in fetching output of task '%s': %s: %s",
+                            task, x.__class__.__name__, str(x))
+                        gc3libs.log.debug(
+                            "Ignored error in fetching output of task '%s': %s: %s",
+                            task, x.__class__.__name__, str(x), exc_info=True)
                 if task.execution.state == Run.State.TERMINATED:
                     self._terminated.append(task)
                     self._core.free(task)
@@ -1107,13 +1145,14 @@ class Engine(object):
 
     def fetch_output(self, task, output_dir=None, overwrite=False, **extra_args):
         """
-        Proxy for `Core.fetch_output` (which see).
+        Enqueue task for later output retrieval.
+
+        .. warning:: FIXME
+
+          The `output_dir` and `overwrite` parameters are currently ignored.
+
         """
-        if output_dir is None and self.output_dir is not None:
-            output_dir = os.path.join(self.output_dir, task.persistent_id)
-        if overwrite is None:
-            overwrite = self.fetch_output_overwrites
-        self._core.fetch_output(task, output_dir, overwrite, **extra_args)
+        self.add(task)
 
 
     def kill(self, task, **extra_args):
@@ -1136,3 +1175,18 @@ class Engine(object):
         e.g. LRMS
         """
         self._core.close()
+
+    # Wrapper methods around `Core` to access the backends directly
+    # from the `Engine`.
+
+    @utils.same_docstring_as(Core.select_resource)
+    def select_resource(self, match):
+        return self._core.select_resource(match)
+
+    @utils.same_docstring_as(Core.get_resources)
+    def get_resources(self):
+        return self._core.get_resources()
+
+    @utils.same_docstring_as(Core.get_backend)
+    def get_backend(self, name):
+        return self._core.get_backend(name)
