@@ -36,7 +36,7 @@ in core Python, namely:
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.0.5 version (SVN $Revision: 3506 $)'
+__version__ = '2.1 version (SVN $Revision: 3697 $)'
 
 
 import gc3libs
@@ -250,6 +250,12 @@ class LRMSError(Error):
 class LRMSSubmitError(Error):
     pass
 
+class LRMSSkipSubmissionToNextIteration(RecoverableError):
+    """
+    Tells the Engine to skip submission of a job until the next
+    iteration.
+    """
+    pass
 
 class NoConfigurationFile(FatalError):
     """
@@ -309,6 +315,18 @@ class RecoverableTransportError(RecoverableError):
 class UnrecoverableTransportError(UnrecoverableError):
     pass
 
+class CopyError(TransportError):
+    """
+    Error copying a file from `source` to `destination.
+    """
+    def __init__(self, source, destination, ex):
+        self.source = source
+        self.destination = destination
+        TransportError.__init__(
+            self,
+            "Could not copy '%s' to '%s': %s: %s"
+            % (source, destination, ex.__class__.__name__, str(ex)))
+
 class UnknownJob(Error, ValueError):
     """
     Raised when an operation is attempted on a task, which is
