@@ -3,7 +3,7 @@
 """
 Implementation of the `core` command-line front-ends.
 """
-# Copyright (C) 2009-2013 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2012 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,7 @@ Implementation of the `core` command-line front-ends.
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.0.5 version (SVN $Revision: 3506 $)'
+__version__ = '2.1 version (SVN $Revision: 3697 $)'
 __author__ = "Sergio Maffioletti <sergio.maffioletti@gc3.uzh.ch>, "
 "Riccardo Murri <riccardo.murri@uzh.ch>"
 "Antonio Messina <arcimboldo@gmail.com>"
@@ -35,7 +35,7 @@ import sys
 import os
 import posix
 import tarfile
-from texttable import Texttable
+from prettytable import PrettyTable
 import time
 import types
 import re
@@ -43,12 +43,10 @@ import re
 ## 3rd party modules
 import cli  # pyCLI
 import cli.app
-from parsedatetime import parsedatetime
+import parsedatetime
 
 ## local modules
 from gc3libs import Application, Run
-import gc3libs.application.gamess as gamess
-import gc3libs.application.rosetta as rosetta
 import gc3libs.exceptions
 import gc3libs.cmdline
 import gc3libs.core as core
@@ -276,13 +274,11 @@ GC3Libs internals.
 
         if self.params.tabular:
             # prepare table prettyprinter
-            table = Texttable(0)  # max_width=0 => dynamically resize cells
-            table.set_cols_align(['l'] * (1 + len(only_keys)))
+            table = PrettyTable()
+            table.border=True
             if self.params.header:
-                table.set_deco(Texttable.HEADER)  # also: .VLINES, .HLINES .BORDER
-                table.header(["Job ID"] + only_keys)
-            else:
-                table.set_deco(0)
+                table.field_names = ["Job ID"] + only_keys
+                table.align = 'l'
 
         if self.params.csv:
             csv_output = csv.writer(sys.stdout)
@@ -325,7 +321,7 @@ GC3Libs internals.
                     # with `-v` and above, dump the whole `Application` object
                     utils.prettyprint(app, indent=4, width=width, only_keys=only_keys)
         if self.params.tabular:
-            print(table.draw())
+            print(table)
         failed = len(self.params.args) - ok
         # exit code is practically limited to 7 bits ...
         return utils.ifelse(failed < 127, failed, 126)
@@ -502,8 +498,15 @@ Print job state.
                     jobname = app.jobname
                 else:
                     jobname = ''
-                rows.append([jobid, jobname, app.execution.state, app.execution.info] +
-                            [app.execution.get(name, "N/A") for name in keys])
+
+                key_values = []
+                for name in keys:
+                    if name in app.execution:
+                        key_values.append(app.execution.get(name))
+                    else:
+                        key_values.append(app.get(name, "N/A"))
+                rows.append([jobid, jobname, app.execution.state, app.execution.info] + key_values)
+
             stats[app.execution.state] += 1
             if app.execution.state == Run.State.TERMINATED:
                 if app.execution.returncode == 0:
@@ -536,9 +539,12 @@ Print job state.
 
         if len(rows) > capacity and self.params.verbose == 0:
             # only print table with statistics
-            table = Texttable(0)  # max_width=0 => dynamically resize cells
-            table.set_deco(0)  # also: .VLINES, .HLINES .BORDER
-            table.set_cols_align(['r', 'c', 'r'])
+            table = PrettyTable(['state', 'num/tot', 'num/tot %'])
+            table.header=False
+            table.align['state'] = 'r'
+            table.align['num/tot'] = 'c'
+            table.align['num/tot %'] = 'r'
+
             for state, num in sorted(stats.items()):
                 if (states is None) or (str(state) in states):
                     table.add_row([
@@ -548,12 +554,11 @@ Print job state.
                         ])
         else:
             # print table of job status
-            table = Texttable(0)  # max_width=0 => dynamically resize cells
-            table.set_deco(Texttable.HEADER)  # also: .VLINES, .HLINES .BORDER
-            table.set_cols_align(['l'] * (4 + len(keys)))
-            table.header(["JobID", "Job name", "State", "Info"] + keys)
-            table.add_rows(sorted(rows), header=False)
-        print(table.draw())
+            table = PrettyTable(["JobID", "Job name", "State", "Info"] + keys)
+            table.align = 'l'
+            for row in sorted(rows):
+                table.add_row(row)
+        print(table)
 
         if self.params.lifetimes is not None and len(lifetimes_rows) > 1:
             if self.params.lifetimes is sys.stdout:
@@ -848,10 +853,11 @@ List status of computational resources.
             return cmp(x.name, y.name)
 
         for resource in sorted(resources, cmp=cmp_by_name):
-            table = Texttable(0)  # max_width=0 => dynamically resize cells
-            table.set_deco(Texttable.HEADER | Texttable.BORDER)  # also: .VLINES, .HLINES
-            table.set_cols_align(['r', 'l', 'l'])
-            table.header(['', resource.name, ''])
+            if self.params.args and resource.name not in self.params.args:
+                continue
+            table = PrettyTable(['', resource.name, ' '])
+            table.align = 'l'
+            table.align[''] = 'r'
 
             # not all resources support the same keys...
             def output_if_exists(name, print_name):
@@ -870,7 +876,8 @@ List status of computational resources.
             output_if_exists('max_memory_per_core', "Max memory per core")
             output_if_exists('max_walltime', "Max walltime per job")
             output_if_exists('applications', "Supported applications")
-            print(table.draw())
+            print(table)
+            print('')
 
 
 class cmd_gsession(_BaseCmd):
@@ -883,7 +890,7 @@ Usage:
 
 commands are listed below, under `subcommands`.
 
-To get detaileid info on a specific command, run:
+To get detailed info on a specific command, run:
 
     gsession `command` --help
     """
@@ -902,7 +909,7 @@ To get detaileid info on a specific command, run:
 
         self.subparsers = self.argparser.add_subparsers(
             title="subcommands",
-            description="gsession accept the the following subcommands. "
+            description="gsession accept the following subcommands. "
             "Each subcommand requires a `SESSION` directory as argument.")
 
         self._add_subcmd(
@@ -939,39 +946,40 @@ To get detaileid info on a specific command, run:
         """
         Called with subcommand `abort`.
 
-        This method will open
-        the desired session and will kill all the jobs which belongs
-        to that session and remove them from store.
+        This method will open the desired session and will kill all
+        the jobs which belongs to that session.
+
+        Thiw method will return the number of jobs that have not been
+        correctly aborted. If this number is greater than 125, then
+        125 will be returned instead, since numbers above 125 have a
+        special meaning for the shell.
         """
         try:
             self.session = Session(self.params.session, create=False)
         except gc3libs.exceptions.InvalidArgument, ex:
             raise RuntimeError('Session %s not found. Please specify a valid session directory as argument' % self.params.session)
 
-        rc = 0
+        rc = len(self.session.tasks)
         for task_id in self.session.tasks.keys():
             task = self.session.tasks[task_id]
             task.attach(self._core)
             if task.execution.state == Run.State.TERMINATED:
                 gc3libs.log.info("Not aborting '%s' which is already in TERMINATED state." % task)
-                rc += 1
+                rc -= 1
                 continue
             try:
                 task.kill()
-            except Exception, ex:
-                gc3libs.log.error(
-                    "Failed killing job '%s'"
-                    "Error type %s, message %s" % (ex.__class__, ex))
-            task.free()
-            rc += 1
-            self.session.remove(task_id)
+            finally:
+                task.free()
+                rc -= 1
 
-        if self.session.tasks:
-            gc3libs.log.error("Not all tasks have been removed from the session.")
+        if rc:
+            gc3libs.log.error("Not all tasks of the session have been aborted.")
 
         if rc > 125:
             # 126 and 127 error codes have special meanings.
             rc = 125
+        self.session.save_all()
         return rc
 
     def delete_session(self):
@@ -981,9 +989,9 @@ To get detaileid info on a specific command, run:
         This method will first call `abort` and then remove the
         current session.
 
-        If the `abort` will fail or not all tasks have been removed
-        from the session, it will not delete the session and will exit
-        with exit value `1`.
+        If the `abort` will fail or not all tasks are in TERMINATED
+        state, it will not delete the session and will exit with the
+        exit status of the `abort`.
         """
         try:
             rc = self.abort_session()
@@ -992,9 +1000,6 @@ To get detaileid info on a specific command, run:
         except gc3libs.exceptions.InvalidArgument, ex:
             raise RuntimeError('Session %s not found' % self.params.session)
 
-        # XXX: FIXME: self.session is created in abort_session()
-        if self.session.tasks:
-            raise RuntimeError("Session %s not empty: not deleting it" % self.params.session)
         self.session.destroy()
         return 0
 
@@ -1016,8 +1021,8 @@ To get detaileid info on a specific command, run:
                 jobname = app.jobname
             except AttributeError:
                 jobname = ''
-
-            rows.append([indent + app.persistent_id,
+            
+            rows.append([indent + str(app.persistent_id),
                          jobname,
                          app.execution.state,
                          app.execution.info])
@@ -1030,12 +1035,11 @@ To get detaileid info on a specific command, run:
         rows = []
         for app in self.session.tasks.values():
             rows.extend(print_app_table(app, '', self.params.recursive))
-        table = Texttable(0)  # max_width=0 => dynamically resize cells
-        table.set_deco(Texttable.HEADER)  # also: .VLINES, .HLINES .BORDER
-        table.set_cols_align(['l'] * 4)
-        table.header(["JobID", "Job name", "State", "Info"])
-        table.add_rows(rows, header=False)
-        print(table.draw())
+        table = PrettyTable(["JobID", "Job name", "State", "Info"])
+        table.align = 'l'
+        for row in rows:
+            table.add_row(row)
+        print(table)
 
     def show_log(self):
         """
@@ -1298,3 +1302,221 @@ Select job IDs based on specific criteria
         # Print remaining job IDs
         if current_jobs:
             print str.join(" ", [str(job.persistent_id) for job in current_jobs])
+
+class cmd_gcloud(_BaseCmd):
+    """
+`gcloud` manage VMs created by the EC2 backend
+
+Usage:
+
+    gcloud `command` [options]
+
+commands are listed below, under `subcommands`.
+
+To get detailed info on a specific command, run:
+
+    gcloud `command` --help
+    """
+
+    def _add_subcmd(self, name, func, help=None):
+        subparser = self.subparsers.add_parser(name, help=help)
+        subparser.set_defaults(func=func)
+        subparser.add_argument('-v', '--verbose', action='count')
+        return subparser
+
+    def setup(self):
+        gc3libs.cmdline._Script.setup(self)
+
+        self.subparsers = self.argparser.add_subparsers(
+            title="subcommands",
+            description="gcloud accept the following subcommands.")
+    
+        listparser = self._add_subcmd(
+            'list',
+            self.list_vms,
+            help='List VMs currently know to the EC2 backend.')
+        listparser.add_argument(
+            '-r', '--resource', metavar="NAME", dest="resource_name",
+            default=None, help="Select resource by name.")
+        listparser.add_argument(
+            '-n', '--no-update', action="store_false", dest="update",
+            help="Do not update job statuses; only print what's in the "
+            "local database.")
+        terminateparser = self._add_subcmd(
+            'terminate',
+            self.terminate_vm,
+            help='Terminate a VM.')
+        terminateparser.add_argument('ID', help='ID of the VM to terminate.', nargs='+')
+        terminateparser.add_argument(
+            '-r', '--resource', metavar="NAME", dest="resource_name",
+            default=None, help="Select resource by name.")
+
+        forgetparser = self._add_subcmd(
+            'forget',
+            self.forget_vm,
+            help="Remove the VM from the list of known VMs, so that the EC2 "
+            "backend will stop submitting jobs on it. Please note that if you "
+            "`forget` a VM, it will disappear from the output of `gcloud list`.")
+        forgetparser.add_argument('ID', help='ID of the VM to "forget".', nargs='+')
+        forgetparser.add_argument(
+            '-r', '--resource', metavar="NAME", dest="resource_name",
+            default=None, help="Select resource by name.")
+        
+        runparser = self._add_subcmd(
+            'run',
+            self.create_vm,
+            help='Run a VM.')
+        runparser.add_argument(
+            '-r', '--resource', metavar="NAME", dest="resource_name",
+            default=None, help="Select resource by name.")
+        runparser.add_argument(
+            '-i', '--image-id', metavar="ID", dest="image_id", default=None,
+            help="Select the image id to use, if different from the one "
+            "specified in the configuration file.")
+
+    def setup_args(self):
+        # prevent GC3UtilsScript.setup_args() to add the default JOBID
+        # non optional argument
+        pass
+
+
+    def main(self):
+        import gc3utils.commands
+
+        resources = [res for res in self._core.get_resources()
+                     if res.type.startswith('ec2')]
+        if self.params.resource_name:
+            resources = [res for res in resources 
+                         if res.name == self.params.resource_name]
+            if not resources:
+                raise RuntimeError('No EC2 resource found matching name `%s`.'
+                                   '' % self.params.resource_name)
+
+        if not resources:
+            raise RuntimeError('No EC2 resource found.')
+
+        self.resources = resources
+
+        return self.params.func()
+
+    def _print_vms(self, vms, res, header=True):
+            table = PrettyTable()
+            table.border=True
+            if header:
+                table.field_names = ["id", "state", "public ip", "Nr. of jobs", "image id", "keypair"]
+            for vm in vms:
+                remote_jobs = 'N/A'
+                if vm.id in res.resources:
+                    if res.resources[vm.id].updated:
+                        remote_jobs = str(len(res.resources[vm.id].job_infos))
+                table.add_row((vm.id, vm.state, vm.public_dns_name, remote_jobs, vm.image_id, vm.key_name))
+            print(table)
+    
+
+    # Subcommand methods
+
+    def list_vms(self):
+        for res in self.resources:
+            printed = 0
+            if self.params.update:
+                res.get_resource_status()
+            else:
+                res._connect()
+            resname = "VMs running on EC2 resource `%s`" % res.name
+            print ("""
+%s
+%s
+%s
+""" % ("="*len(resname), resname, "="*len(resname)))
+
+            vms = res._vms.get_all_vms()
+            if vms:
+                self._print_vms(vms, res)
+                printed += len(vms)
+
+            if not printed:
+                print "  no known VMs are currently running on this resource."
+
+        return 0
+
+    def _find_resources_by_running_vm(self, vmid):
+        """
+        Returns a list of resources that are currently running a VM
+        with id `vmid`
+        """
+        matching_res = []
+        for resource in self.resources:
+            if vmid in resource._vms:
+                matching_res.append(resource)
+        return matching_res
+
+    def _terminate_vm(self, vmid):
+        matching_res = self._find_resources_by_running_vm(vmid)
+        if len(matching_res) > 1:
+            raise LookupError("VM with ID `%s` have been found on multiple "
+                               "resources. Please specify the resource by "
+                               "running `grun` with the `-r` option.")
+        elif not matching_res:
+            raise LookupError(
+                "VM with id `%s` not found." % (vmid))
+
+        resource = matching_res[0]
+        vm = resource._vms.get_vm(vmid)
+        gc3libs.log.info("Terminating VM `%s` on resource `%s`" %
+                         (vmid, resource.name))
+        vm.terminate()
+        resource._vms.remove_vm(vmid)
+        resource._session.save(resource._vms)
+
+    def terminate_vm(self):
+        for resource in self.resources:
+            resource.get_resource_status()
+        errors = 0
+        for vmid in self.params.ID:
+            try:
+                self._terminate_vm(vmid)
+            except LookupError, ex:
+                gc3libs.log.warning(str(ex))
+                errors += 1
+        return errors
+
+    def _forget_vm(self, vmid):
+        matching_res = self._find_resources_running_vm(vmid)
+        if len(matching_res) > 1:
+            raise LookupError("VM with ID `%s` have been found on multiple "
+                               "resources. Please specify the resource by "
+                               "running `grun` with the `-r` option.")
+        elif not matching_res:
+            raise LookupError(
+                "VM with id `%s` not found." % (vmid))
+
+        resource = matching_res[0]
+        resource._vms.remove_vm(vmid)
+        resource._session.save(resource._vms)
+        
+
+    def forget_vm(self):
+        for resource in self.resources:
+            # calling `_connect()` will load the list of available VMs
+            resource._connect()
+        errors = 0
+        for vmid in self.params.ID:
+            try:
+                self._forget_vm(vmid)
+            except LookupError, ex:
+                gc3libs.log.warning(str(ex))
+                errors += 1
+        return errors
+
+    def create_vm(self):
+        if len(self.resources) > 1:
+            raise RuntimeError("Please specify the resource where you want to "
+                               "create the VM by supplying the `-r` option.")
+
+        resource = self.resources[0]
+        resource._connect()
+        image_id = self.params.image_id or resource.image_id 
+        vm = resource._create_instance(image_id)
+        resource._vms.add_vm(vm)
+        resource._session.save(resource._vms)
+        self._print_vms([vm], resource)
